%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab routine for Binomial-Neighborhood Algorithm
%%%  
%%% Inputs:   genes       := genes to predict for, entries being indices of genes in gene_list, (# genes) x 1
%%%           terms       := GO terms in the hierarchy to predict for, entries being indices of terms 
%%%                          in go_by_go, first entry is the root, (# terms) x 1
%%%           train       := genes used for training with entries being indices in ppi, (# train) x 1
%%%           ppi         := protein-protein interaction, 5143 x 5143
%%%           gene_by_go  := gene-GO association matrix, 5143 x 1927
%%% Outputs:  f           := probabilities of labels of terms on genes, (# genes) x (# terms)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = bn_prob( genes, terms, train, ppi, gene_by_go )

e = 10^(-6);

for s = 1 : length(genes)
    for t = 2 : length(terms)
        
        y = gene_by_go(:, terms(t));
        neighbor = train(find(ppi(train, genes(s)) == 1));

        k = length(find(y(neighbor) == 1));
        N = length(neighbor);

        %% Estimate f1 = P(a protein has term t) by the training set
        f1 = length( find(y(train) == 1) )/ length(train) ;
        f2 = 1 - f1;

        if f1 == 0
            f(s, t-1) = 0;
            continue
        else

            %% Estimate p1, p0 by MLE
            % p1 = P(a neighbor has term t | gene s has term t)
            % p0 = P(a neighbor has term t | gene s DO NOT have term t)
            ppi_tr = ppi(train, train);
    
            A = 0; % # of ch-ch edges 
            B = 0; % # of ch-pa edges
            C = 0; % # of pa-pa edges

            for i = 1 : length(train)
                neigh_tr = find(ppi_tr(i, :) == 1);
                if length(neigh_tr) > 0
                    for j = 1 : length(neigh_tr)
                        if gene_by_go(train(i), terms(t)) + gene_by_go(train(neigh_tr(j)), terms(t)) == 2
                            A = A + 1; % a ch-ch edge
                            C = C + 1;
                        elseif gene_by_go(train(i), terms(t)) + gene_by_go(train(neigh_tr(j)), terms(t)) == 0
                            B = B + 1; % a ch-pa edge
                        else
                            C = C + 1; % a pa-pa edge
                        end;
                    end;
                end;      
            end;

            if A + B == 0 || B + C == 0
                f(s, t-1) = 0; 
                continue
            else
                p1 = 2*A/(2*A + B);
                p0 = B/(2*C + B);
                
                % Compute probability f for gene s
                a = log(binopdf(k, N, p1) + e) + log(f1 + e);
                b = log(binopdf(k, N, p0) + e) + log(f2 + e);
                f(s, t-1) = exp(a) / ( exp(a) + exp(b) );
               
            end;
        end;
        
    end;
end;
