%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% MatLab routine for Hierarchical Binomial-Neighborhood Algorithm on ONE
%%%        gene and ONE pair of parent and child terms
%%%
%%% Inputs:    ch          := child GO term, value being the index of the term in go_list, scalar
%%%            pa          := parent GO term, value being the index of the term in go_list, scalar
%%%            gene        := gene to predict for with value being the index of the gene in gene_list , scalar
%%%            train       := genes used for training with entries being indices in ppi, (# train) x 1
%%%            ppi         := protein-protein interaction, 5143 x 5143
%%%            gene_by_go  := gene-GO association matrix, 5143 x 1927
%%% Outputs:   f           := probability that gene is labeled with the child term, scalar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = hbn_prob_one( ch, pa, gene, train, ppi, gene_by_go )


e = 10^(-6);
% gene's neighbors in the training set
neighbor = train(find(ppi(train, gene) == 1)); 

k_ch = length(find( gene_by_go(neighbor, ch) == 1 )); 
k_pa = length(find( gene_by_go(neighbor, pa) == 1 ));

if length( find( gene_by_go(train, pa) == 1 ) ) == 0
   % If nobody in the training set is associated with parent GO term
   f = 0;
else
    %% Estimate f1 = P(a protein has ch | it has pa ) by the training set
   f1 = length( find(gene_by_go(train, ch) == 1) )/ length( find( gene_by_go(train, pa) == 1 ) );
   f2 = 1 - f1;

   if f1 == 0
       f = 0;
   else

       %% Estimate p1, p0 by MLE
       % p1 = P(a neighbor has ch and pa | gene has ch and pa)
       % p0 = P(a neighbor has ch and pa | gene DO NOT have ch but does have pa)
       A = 0; % # of ch-ch edges 
       B = 0; % # of ch-pa edges
       C = 0; % # of pa-pa edges

       %%% Delete genes NOT labeled with parent GO term
       %%% They are automatically NOT labeled with the child GO term
       genes_to_delete = find(gene_by_go(train, pa) == -1);

       %%% Recover the ppi with genes labeled with at least the parent GO term
       if length(genes_to_delete) > 0
           for i = 1 : length(genes_to_delete)
               train(genes_to_delete(i)) = 0;
           end;
           train = train( find(train) );
       end;
       if length(train) == 0 
           f = 0;
           return;
       else
           ppi_tr = ppi(train, train); 
       end;

       if sum(sum(ppi_tr)) == 0
          f = 0;  
          return
       else
          for i = 1 : length(train)
              neigh_tr = find(ppi_tr(i, :) == 1);
              if length(neigh_tr) > 0
                 for j = 1 : length(neigh_tr)
                     if gene_by_go(train(i), ch) + gene_by_go(train(neigh_tr(j)), ch) == 2
                        A = A + 1; % a ch-ch edge
                        C = C + 1; % it is also a pa-pa edge
                     elseif gene_by_go(train(i), ch) + gene_by_go(train(neigh_tr(j)), ch) == 0
                        B = B + 1; % a ch-pa edge
                     else
                        C = C + 1; % a pa-pa edge
                     end;
                 end;
              end;
           end;
       end;

       if A + B == 0 || B + C == 0
          f = 0; 
          return
       else
          p1 = 2*A/(2*A + B);
          p0 = B/(2*C + B);
       end;
       % Compute the probability
       a = log(binopdf(k_ch, k_pa, p1) + e) + log(f1 + e);
       b = log(binopdf(k_ch, k_pa, p0) + e) + log(f2 + e);
       f = exp(a) / ( exp(a) + exp(b) );
   end;
end;
      
