%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab Routine to generate hF measure plot
%%%
%%%   Inputs:
%%%           threshold   := threshold applied to f, f >= threshold, prediction = 1; ow, prediction = -1, vector
%%%           Fold        := number of folds in cross validation, scalar 
%%%           terms       := GO terms from a given hierarchy that are prediected for, entries being indices of 
%%%                          the terms in go_list, vector
%%%           f           := probabilities from a certain method, (# terms) x (# genes as defined in the routine)
%%%           ppi         := protein-protein interaction, 5143 x 5143
%%%           gene_by_go  := gene-GO association matrix, 5143 x 1927
%%%           go_by_go    := go hierarchical structure matrix, entry (i, j) = 1 indicating that term j is an 
%%%                          ancestor of term j, 1927 x 1927
%%%   Outputs:
%%%           plot of threshold vs hF measure over a given GO hierarchy based on all genes predicted
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   

function hF_plot(threshold, Fold, terms, f, ppi, gene_by_go, go_by_go)

root = terms(1);
desc = terms(2 : length(terms))';
genes = find(gene_by_go(:, root) == 1);
true = gene_by_go(genes, desc);
for i = 1 : length(genes)
    a(i) = length(find(gene_by_go(genes(i), desc) == 1));
end;

for fold = 1 : Fold
    if fold < Fold
        id = 1 + (fold - 1) * floor(length(sum(ppi))/Fold) : fold * floor(length(sum(ppi))/Fold);
        n(fold) = length(id(find(gene_by_go(id, root) == 1)));
        s(fold) = sum(n(1 : fold));
                
    elseif fold == Fold
        id = 1 + (fold - 1) * floor(length(sum(ppi))/Fold) : length(sum(ppi));
        n(fold) = length(id(find(gene_by_go(id, root) == 1)));
        s(fold) = sum(n(1 : fold));
    end;
end;

for k = 1 : length(threshold)
    prediction = sign(f - threshold(k));
    prediction(find(prediction == 0)) = ones(length(find(prediction == 0)), 1);
    prediction = propagation(prediction, terms, go_by_go);
       
    for j = 1 : length(genes) 
        c(j) = length(find(prediction(:, j) + true(j, :)' == 2));
        b(j) = length(find(prediction(:, j) == 1));
    end;
       
    C(1) = sum(c(1 : s(1)));
    B(1) = sum(b(1 : s(1)));
    A(1) = sum(a(1 : s(1)));
    if B(1) + A(1) == 0
        hf(k, 1) = 0;
    else
        hf(k, 1) = 2*C(1)/(B(1) + A(1));
    end;
       
    for j = 2 : 5
        C(j) = sum(c(s(j - 1) + 1 : s(j)));
        B(j) = sum(b(s(j - 1) + 1 : s(j)));
        A(j) = sum(a(s(j - 1) + 1 : s(j)));
        if B(j) + A(j) == 0
            hf(k, j) = 0;
        else
            hf(k, j) = 2*C(j)/(B(j) + A(j));
        end;
    end;

    y(k) = mean(hf(k, :));
end;

hold on;
plot(threshold, y, '-r*', 'Markersize', 5) 
xlabel('Threshold for prediction', 'FontSize', 15)
ylabel('hF Measure over Genes', 'FontSize', 15)
hold off;